/* gdevrpdl.c */
/* RICOH RPDL driver */
#include "gdevprn.h"

#define LINE_SIZE ((600 * 2970 / 254) / 8)	/* bytes per line */

/* The device descriptors */
/* paper size A3 */
private dev_proc_print_page(rpdlA3_print_page);

gx_device_printer far_data gs_rpdlA3_device =
  prn_device(prn_std_procs, "rpdlA3",
	29700/254, 42000/254, 600, 600,	/* width*10, height*10, x_dpi, y_dpi */
	0.20, 0.20, 0.20, 0.20,				/* margins */
	1, rpdlA3_print_page);

#define A4toA3_MAG 1.414

gx_device_printer far_data gs_rpdlA4toA3_device =
  prn_device(prn_std_procs, "rpdlA4toA3",
	21000/254, 29700/254, 600*A4toA3_MAG, 600*A4toA3_MAG,	/* width*10, height*10, x_dpi, y_dpi */
	0.20/A4toA3_MAG, 0.20/A4toA3_MAG, 0.20/A4toA3_MAG, 0.20/A4toA3_MAG,		/* margins */
	1, rpdlA3_print_page);

/* paper size A4 */
private dev_proc_print_page(rpdlA4_print_page);

gx_device_printer far_data gs_rpdlA4_device =
  prn_device(prn_std_procs, "rpdlA4",
	21000/254, 29700/254, 600, 600,	/* width*10, height*10, x_dpi, y_dpi */
	0.20, 0.20, 0.20, 0.20,				/* margins */
	1, rpdlA4_print_page);

/* paper size B4 */
private dev_proc_print_page(rpdlB4_print_page);

gx_device_printer far_data gs_rpdlB4_device =
  prn_device(prn_std_procs, "rpdlB4",
	25700/254, 36400/254, 600, 600,	/* width*10, height*10, x_dpi, y_dpi */
	0.20, 0.20, 0.20, 0.20,		/* margins */
	1, rpdlB4_print_page);

#define A4toB4_MAG 1.225

gx_device_printer far_data gs_rpdlA4toB4_device =
  prn_device(prn_std_procs, "rpdlA4toB4",
	21000/254, 29700/254, 600*A4toB4_MAG, 600*A4toB4_MAG,	/* width*10, height*10, x_dpi, y_dpi */
	0.20/A4toB4_MAG, 0.20/A4toB4_MAG, 0.20/A4toB4_MAG, 0.20/A4toB4_MAG,		/* margins */
	1, rpdlB4_print_page);

/* paper size B5 */
private dev_proc_print_page(rpdlB5_print_page);

gx_device_printer far_data gs_rpdlB5_device =
  prn_device(prn_std_procs, "rpdlB5",
	18200/254, 25700/254, 600, 600,	/* width*10, height*10, x_dpi, y_dpi */
	0.20, 0.20, 0.20, 0.20,		/* margins */
	1, rpdlB5_print_page);

/* ------ Internal routines ------ */

#define	SUB 0x1a
#define ESC 0x1b
#define	DC2 0x12
#define	EM  0x19

/* Send the page to the printer.  */
private int
rpdl_print_page(gx_device_printer *pdev, FILE *pstream)
{
	char row_data[LINE_SIZE*2];
	char comp_data[LINE_SIZE*2];
	int lnum, line_size = gdev_prn_raster(pdev);

	for ( lnum = 0; lnum < pdev->height; lnum++ ) {
		char *bgn_data;
		char *end_data;

		gdev_prn_copy_scan_lines(pdev, lnum, (byte *)row_data, line_size);
		bgn_data = row_data;
		end_data = row_data + line_size;

		/* Remove leading 0s. */
		while ( end_data > bgn_data && bgn_data[0] == 0 ) bgn_data++;

		/* Remove trailing 0s. */
		while ( end_data > bgn_data && end_data[-1] == 0 ) end_data--;

		if ( end_data != bgn_data ) {
			int	num_comp = 0, num_row = end_data - bgn_data;
			int	c = 0, count = 0;
			char	*rp, *cp;

			/* compress */
			for(rp = bgn_data, cp = comp_data; rp < end_data; rp++){
				if((*rp == c) && (count < 256)){
					if(count == 0){
						*cp++ = *rp;
					}
					count++;
				}else{
					if(count){
						*cp++ = count - 1;
						count = 0;
					}
					*cp++ = c = *rp;
				}
			}
			if(count) *cp++ = count - 1;
			num_comp = cp - comp_data;

			/* move */
			fprintf(pstream, "%c%cG3,%d,1,1,4,%d,%d,%d@", ESC, DC2, (end_data - bgn_data)*8, (bgn_data - row_data)*8, lnum, num_comp);

			/* send the row */
			fwrite(comp_data, sizeof(char), num_comp, pstream);
		}
	}

	/* print page */
	fputc(0x0C, pstream);

	return 0;
}

private int
rpdlA3_print_page(gx_device_printer *pdev, FILE *pstream)
{
	static const char RpdlInit[] = {
		ESC, SUB, 'I',					/* printer reset */
		ESC, DC2, '!', '@', 'R', '0', '0', ESC, ' ',	/* RPDL mode */
		ESC, DC2, 'Y', 'W', ',', '3', ' ',		/* graphics (600dpi) */
		ESC, DC2, 'Y', 'A', '0', '4', ',', '3', ' ',	/* engine (600dpi) */
		ESC, DC2, 'M', '1', ' ',			/* normal CR,LF,FF */
		ESC, '!',					/* auto CR-LF disable */
		ESC, EM,  'M',                 /* selecting multi feeder tray */
		ESC, DC2, '3', '3', ' ',		/* force exhausting time */
		ESC, DC2, '5', '2', '@', 'A', '3', 'R', ESC, ' ',	/* paper size A3 */
		ESC, DC2, 'D', '1', ' ',			/* portlait */
		ESC, DC2, '#', '4', ' ',/* coordination unit(600dpi) */
		ESC, DC2, 'Q', '5', ' ',/* spaceing unit(600dpi) */
	};
	fwrite(RpdlInit, sizeof(RpdlInit), 1, pstream);		/* initialize */
	return rpdl_print_page(pdev, pstream);
}

private int
rpdlA4_print_page(gx_device_printer *pdev, FILE *pstream)
{
	static const char RpdlInit[] = {
		ESC, SUB, 'I',					/* printer reset */
		ESC, DC2, '!', '@', 'R', '0', '0', ESC, ' ',	/* RPDL mode */
		ESC, DC2, 'Y', 'W', ',', '3', ' ',		/* graphics (600dpi) */
		ESC, DC2, 'Y', 'A', '0', '4', ',', '3', ' ',	/* engine (600dpi) */
		ESC, DC2, 'M', '1', ' ',			/* normal CR,LF,FF */
		ESC, '!',					/* auto CR-LF disable */
		ESC, DC2, '3', '3', ' ',		/* force exhausting time */
		ESC, EM,  'M',                 /* selecting multi feeder tray */
		ESC, DC2, '5', '2', '@', 'A', '4', 'R', ESC, ' ',	/* paper size A4 */
		ESC, DC2, 'D', '1', ' ',			/* portlait */
		ESC, DC2, '#', '4', ' ',/* coordination unit(600dpi) */
		ESC, DC2, 'Q', '5', ' ',/* spaceing unit(600dpi) */
	};
	fwrite(RpdlInit, sizeof(RpdlInit), 1, pstream);		/* initialize */
	return rpdl_print_page(pdev, pstream);
}

private int
rpdlB4_print_page(gx_device_printer *pdev, FILE *pstream)
{
	static const char RpdlInit[] = {
		ESC, SUB, 'I',					/* printer reset */
		ESC, DC2, '!', '@', 'R', '0', '0', ESC, ' ',	/* RPDL mode */
		ESC, DC2, 'Y', 'W', ',', '3', ' ',		/* graphics (600dpi) */
		ESC, DC2, 'Y', 'A', '0', '4', ',', '3', ' ',	/* engine (600dpi) */
		ESC, DC2, 'M', '1', ' ',			/* normal CR,LF,FF */
		ESC, '!',					/* auto CR-LF disable */
		ESC, DC2, '3', '3', ' ',		/* force exhausting time */
		ESC, EM,  'M',                 /* selecting multi feeder tray */
		ESC, DC2, '5', '2', '@', 'B', '4', 'R', ESC, ' ',	/* paper size B4 */
		ESC, DC2, 'D', '1', ' ',			/* portlait */
		ESC, DC2, '#', '4', ' ',/* coordination unit(600dpi) */
		ESC, DC2, 'Q', '5', ' ',/* spaceing unit(600dpi) */
	};
	fwrite(RpdlInit, sizeof(RpdlInit), 1, pstream);		/* initialize */
	return rpdl_print_page(pdev, pstream);
}

private int
rpdlB5_print_page(gx_device_printer *pdev, FILE *pstream)
{
	static const char RpdlInit[] = {
		ESC, SUB, 'I',					/* printer reset */
		ESC, DC2, '!', '@', 'R', '0', '0', ESC, ' ',	/* RPDL mode */
		ESC, DC2, 'Y', 'W', ',', '3', ' ',		/* graphics (600dpi) */
		ESC, DC2, 'Y', 'A', '0', '4', ',', '3', ' ',	/* engine (600dpi) */
		ESC, DC2, 'M', '1', ' ',			/* normal CR,LF,FF */
		ESC, '!',					/* auto CR-LF disable */
		ESC, DC2, '3', '3', ' ',		/* force exhausting time */
		ESC, EM,  'M',                 /* selecting multi feeder tray */
		ESC, DC2, '5', '2', '@', 'B', '5', 'R', ESC, ' ',	/* paper size B5 */
		ESC, DC2, 'D', '1', ' ',			/* portlait */
		ESC, DC2, '#', '4', ' ',/* coordination unit(600dpi) */
		ESC, DC2, 'Q', '5', ' ',/* spaceing unit(600dpi) */
	};
	fwrite(RpdlInit, sizeof(RpdlInit), 1, pstream);		/* initialize */
	return rpdl_print_page(pdev, pstream);
}

